@@ -1,7 +1,7 @@
 # This script configures QScintilla for PyQt v4.10 and later.  It will fall
 # back to the old script if an earlier version of PyQt is found.
 #
-# Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of QScintilla.
 # 
@@ -38,9 +38,9 @@ except ImportError:
 # Initialise the constants.
 SIP_MIN_VERSION = '4.12.0'
 
-# This must be kept in sync with qscintilla.pro, Qt4Qt5/application.pro and
-# Qt4Qt5/designer.pro.
-QSCI_API_MAJOR = 9
+# This must be kept in sync with Python/configure-old.py, qscintilla.pro,
+# example-Qt4Qt5/application.pro and designer-Qt4Qt5/designer.pro.
+QSCI_API_MAJOR = 11
 
 
 def error(msg):
@@ -315,8 +315,10 @@ class TargetConfiguration:
                 self.qmake_spec = 'macx-g++'
 
         self.api_dir = qt_config.QT_INSTALL_DATA
-        self.qsci_inc_dir = qt_config.QT_INSTALL_HEADERS
-        self.qsci_lib_dir = qt_config.QT_INSTALL_LIBS
+        self.qsci_default_inc_dir = qt_config.QT_INSTALL_HEADERS
+        self.qsci_inc_dir = None
+        self.qsci_default_lib_dir = qt_config.QT_INSTALL_LIBS
+        self.qsci_lib_dir = None
 
     def override_defaults(self, opts):
         """ Override the defaults from the command line.  opts are the command
@@ -404,7 +406,7 @@ def create_optparser(target_config):
         setattr(parser.values, option.dest, os.path.abspath(value))
 
     p = optparse.OptionParser(usage="python %prog [options]",
-            version="2.7.2")
+            version="2.8.1")
 
     p.add_option("--spec", dest='qmakespec', default=None, action='store',
             metavar="SPEC",
@@ -526,10 +528,14 @@ def check_qscintilla(target_config):
     """
 
     # Find the QScintilla header files.
-    sciglobal = os.path.join(target_config.qsci_inc_dir, 'Qsci', 'qsciglobal.h')
+    inc_dir = target_config.qsci_inc_dir
+    if inc_dir is None:
+        inc_dir = target_config.qsci_default_inc_dir
+
+    sciglobal = os.path.join(inc_dir, 'Qsci', 'qsciglobal.h')
 
     if not os.access(sciglobal, os.F_OK):
-        error("Qsci/qsciglobal.h could not be found in %s. If QScintilla is installed then use the --qsci-incdir argument to explicitly specify the correct directory." % target_config.qsci_inc_dir)
+        error("Qsci/qsciglobal.h could not be found in %s. If QScintilla is installed then use the --qsci-incdir argument to explicitly specify the correct directory." % inc_dir)
 
     # Get the QScintilla version string.
     sciversstr = read_define(sciglobal, 'QSCINTILLA_VERSION_STR')
@@ -538,14 +544,18 @@ def check_qscintilla(target_config):
                 "The QScintilla version number could not be determined by "
                 "reading %s." % sciglobal)
 
-    if not glob.glob(os.path.join(target_config.qsci_lib_dir, '*qscintilla2*')):
-        error("The QScintilla library could not be found in %s. If QScintilla is installed then use the --qsci-libdir argument to explicitly specify the correct directory." % target_config.qsci_lib_dir)
+    lib_dir = target_config.qsci_lib_dir
+    if lib_dir is None:
+        lib_dir = target_config.qsci_default_lib_dir
+
+    if not glob.glob(os.path.join(lib_dir, '*qscintilla2*')):
+        error("The QScintilla library could not be found in %s. If QScintilla is installed then use the --qsci-libdir argument to explicitly specify the correct directory." % lib_dir)
 
     # Because we include the Python bindings with the C++ code we can
     # reasonably force the same version to be used and not bother about
     # versioning.
-    if sciversstr != '2.7.2':
-        error("QScintilla %s is being used but the Python bindings 2.7.2 are being built. Please use matching versions." % sciversstr)
+    if sciversstr != '2.8.1':
+        error("QScintilla %s is being used but the Python bindings 2.8.1 are being built. Please use matching versions." % sciversstr)
 
     inform("QScintilla %s is being used." % sciversstr)
 
@@ -660,12 +670,7 @@ def generate_pro(target_config, opts):
     pro.write('TEMPLATE = lib\n')
     pro.write('CONFIG += %s\n' % ('debug' if opts.debug else 'release'))
     pro.write('CONFIG += %s\n' % ('staticlib' if opts.static else 'plugin'))
-
-    pro.write('''
-greaterThan(QT_MAJOR_VERSION, 4) {
-    QT += widgets printsupport
-}
-''')
+    pro.write('CONFIG += qscintilla2\n')
 
     if not opts.static:
         # I don't really understand why the linker needs to find the Python
@@ -730,12 +735,18 @@ INSTALLS += sip
     if target_config.qscintilla_is_dll:
         pro.write('DEFINES += QSCINTILLA_DLL\n')
 
-    pro.write('INCLUDEPATH += %s\n' % target_config.qsci_inc_dir)
+    if target_config.qsci_inc_dir is not None:
+        pro.write('INCLUDEPATH += %s\n' % target_config.qsci_inc_dir)
+
     pro.write('INCLUDEPATH += %s\n' % target_config.py_inc_dir)
     if target_config.py_inc_dir != target_config.sip_inc_dir:
         pro.write('INCLUDEPATH += %s\n' % target_config.sip_inc_dir)
 
-    pro.write('LIBS += -L%s -lqscintilla2\n' % target_config.qsci_lib_dir)
+    lib_dir = target_config.qsci_lib_dir
+    if lib_dir is None:
+        lib_dir = target_config.qsci_default_lib_dir
+    else:
+        pro.write('LIBS += -L%s -lqscintilla2\n' % lib_dir)
 
     if not opts.static:
         pro.write('''
@@ -747,9 +758,10 @@ win32 {
 
 macx {
     QMAKE_LFLAGS += "-undefined dynamic_lookup"
+    QMAKE_LFLAGS += "-install_name $$absolute_path($$PY_MODULE, $$target.path)"
     QMAKE_POST_LINK = $$QMAKE_POST_LINK$$escape_expand(\\\\n\\\\t)$$quote(install_name_tool -change libqscintilla2.%s.dylib %s/libqscintilla2.%s.dylib $$PY_MODULE)
 }
-''' % (QSCI_API_MAJOR, target_config.qsci_lib_dir, QSCI_API_MAJOR))
+''' % (QSCI_API_MAJOR, lib_dir, QSCI_API_MAJOR))
 
     pro.write('\n')
     pro.write('TARGET = Qsci\n')
