@@ -1,6 +1,6 @@
 // This defines the interface to the QsciLexerCSS class.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -120,7 +120,10 @@ public:
         ExtendedPseudoElement = 21,
 
         //! A media rule.
-        MediaRule = 22
+        MediaRule = 22,
+
+        //! A variable.
+        Variable = 23,
     };
 
     //! Construct a QsciLexerCSS with parent \a parent and name \a name.  \a
@@ -147,7 +150,7 @@ public:
     //! auto-indentation.  The styles is returned via \a style.
     const char *blockStart(int *style = 0) const;
 
-    //! \internal Returns the string of characters that comprise a word.
+    //! Returns the string of characters that comprise a word.
     const char *wordCharacters() const;
 
     //! Returns the foreground colour of the text for style number \a style.
@@ -179,6 +182,39 @@ public:
     //! \sa setFoldCompact()
     bool foldCompact() const;
 
+    //! If \a enabled is true then support for HSS is enabled.  The default is
+    //! false.
+    //!
+    //! \sa HSSLanguage()
+    void setHSSLanguage(bool enabled);
+
+    //! Returns true if support for HSS is enabled.
+    //!
+    //! \sa setHSSLanguage()
+    bool HSSLanguage() const {return hss_language;}
+
+    //! If \a enabled is true then support for Less CSS is enabled.  The
+    //! default is false.
+    //!
+    //! \sa LessLanguage()
+    void setLessLanguage(bool enabled);
+
+    //! Returns true if support for Less CSS is enabled.
+    //!
+    //! \sa setLessLanguage()
+    bool LessLanguage() const {return less_language;}
+
+    //! If \a enabled is true then support for Sassy CSS is enabled.  The
+    //! default is false.
+    //!
+    //! \sa SCSSLanguage()
+    void setSCSSLanguage(bool enabled);
+
+    //! Returns true if support for Sassy CSS is enabled.
+    //!
+    //! \sa setSCSSLanguage()
+    bool SCSSLanguage() const {return scss_language;}
+
 public slots:
     //! If \a fold is true then multi-line comment blocks can be folded.
     //! The default is false.
@@ -208,9 +244,15 @@ protected:
 private:
     void setCommentProp();
     void setCompactProp();
+    void setHSSProp();
+    void setLessProp();
+    void setSCSSProp();
 
     bool fold_comments;
     bool fold_compact;
+    bool hss_language;
+    bool less_language;
+    bool scss_language;
 
     QsciLexerCSS(const QsciLexerCSS &);
     QsciLexerCSS &operator=(const QsciLexerCSS &);
