@@ -1,6 +1,6 @@
 // This module defines interface to the QsciStyle class.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -42,6 +37,9 @@ extern "C++" {
 #include <Qsci/qsciglobal.h>
 
 
+class QsciScintillaBase;
+
+
 //! \brief The QsciStyle class encapsulates all the attributes of a style.
 //!
 //! Each character of a document has an associated style which determines how
@@ -72,10 +70,13 @@ public:
     //! Constructs a QsciStyle instance for style number \a style.  If \a style
     //! is negative then a new style number is automatically allocated.  The
     //! styles description, color, paper color, font and end-of-line fill are
-    //! set to \a description, \a color, \a paper, \a font and \a eol_fill
+    //! set to \a description, \a color, \a paper, \a font and \a eolFill
     //! respectively.
     QsciStyle(int style, const QString &description, const QColor &color,
-            const QColor &paper, const QFont &font, bool eol_fill = false);
+            const QColor &paper, const QFont &font, bool eolFill = false);
+
+    //! \internal Apply the style to a particular editor.
+    void apply(QsciScintillaBase *sci) const;
 
     //! Returns the number of the style.
     int style() const {return style_nr;}
