@@ -1,6 +1,6 @@
 // This module implements the QsciLexerHTML class.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -42,7 +37,8 @@
 QsciLexerHTML::QsciLexerHTML(QObject *parent, const char *name)
     : QsciLexer(parent, name),
       fold_compact(true), fold_preproc(true), case_sens_tags(false),
-      fold_script_comments(false), fold_script_heredocs(false)
+      fold_script_comments(false), fold_script_heredocs(false),
+      django_templates(false), mako_templates(false)
 {
 }
 
@@ -67,10 +63,17 @@ const char *QsciLexerHTML::lexer() const
 }
 
 
+// Return the auto-completion fillup characters.
+const char *QsciLexerHTML::autoCompletionFillups() const
+{
+    return "/>";
+}
+
+
 // Return the string of characters that comprise a word.
 const char *QsciLexerHTML::wordCharacters() const
 {
-        return "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-";
+    return "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-";
 }
 
 
@@ -307,6 +310,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case Entity:
 #if defined(Q_OS_WIN)
         f = QFont("Times New Roman",11);
+#elif defined(Q_OS_MAC)
+        f = QFont("Times New Roman", 12);
 #else
         f = QFont("Bitstream Charter",10);
 #endif
@@ -315,6 +320,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case HTMLComment:
 #if defined(Q_OS_WIN)
         f = QFont("Verdana",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Verdana", 12);
 #else
         f = QFont("Bitstream Vera Sans",8);
 #endif
@@ -343,6 +350,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case ASPJavaScriptSymbol:
 #if defined(Q_OS_WIN)
         f = QFont("Comic Sans MS",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Comic Sans MS", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
@@ -368,6 +377,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case PHPComment:
 #if defined(Q_OS_WIN)
         f = QFont("Comic Sans MS",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Comic Sans MS", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
@@ -385,6 +396,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case ASPVBScriptUnclosedString:
 #if defined(Q_OS_WIN)
         f = QFont("Lucida Sans Unicode",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Lucida Grande", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
@@ -394,6 +407,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case ASPVBScriptKeyword:
 #if defined(Q_OS_WIN)
         f = QFont("Lucida Sans Unicode",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Lucida Grande", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
@@ -406,6 +421,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case ASPPythonSingleQuotedString:
 #if defined(Q_OS_WIN)
         f = QFont("Courier New",10);
+#elif defined(Q_OS_MAC)
+        f = QFont("Courier New", 12);
 #else
         f = QFont("Bitstream Vera Sans Mono",9);
 #endif
@@ -421,6 +438,8 @@ QFont QsciLexerHTML::defaultFont(int style) const
     case PHPCommentLine:
 #if defined(Q_OS_WIN)
         f = QFont("Comic Sans MS",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Comic Sans MS", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
@@ -1016,6 +1035,8 @@ void QsciLexerHTML::refreshProperties()
     setCaseSensTagsProp();
     setScriptCommentsProp();
     setScriptHeredocsProp();
+    setDjangoProp();
+    setMakoProp();
 }
 
 
@@ -1061,6 +1082,20 @@ bool QsciLexerHTML::readProperties(QSettings &qs,const QString &prefix)
     else
         rc = false;
 
+    flag = qs.readBoolEntry(prefix + "djangotemplates", false, &ok);
+
+    if (ok)
+        django_templates = flag;
+    else
+        rc = false;
+
+    flag = qs.readBoolEntry(prefix + "makotemplates", false, &ok);
+
+    if (ok)
+        mako_templates = flag;
+    else
+        rc = false;
+
     return rc;
 }
 
@@ -1085,17 +1120,16 @@ bool QsciLexerHTML::writeProperties(QSettings &qs,const QString &prefix) const
     if (!qs.writeEntry(prefix + "foldscriptheredocs", fold_script_heredocs))
         rc = false;
 
+    if (!qs.writeEntry(prefix + "djangotemplates", django_templates))
+        rc = false;
+
+    if (!qs.writeEntry(prefix + "makotemplates", mako_templates))
+        rc = false;
+
     return rc;
 }
 
 
-// Return true if tags are case sensitive.
-bool QsciLexerHTML::caseSensitiveTags() const
-{
-    return case_sens_tags;
-}
-
-
 // Set if tags are case sensitive.
 void QsciLexerHTML::setCaseSensitiveTags(bool sens)
 {
@@ -1112,13 +1146,6 @@ void QsciLexerHTML::setCaseSensTagsProp()
 }
 
 
-// Return true if folds are compact.
-bool QsciLexerHTML::foldCompact() const
-{
-    return fold_compact;
-}
-
-
 // Set if folds are compact
 void QsciLexerHTML::setFoldCompact(bool fold)
 {
@@ -1135,13 +1162,6 @@ void QsciLexerHTML::setCompactProp()
 }
 
 
-// Return true if preprocessor blocks can be folded.
-bool QsciLexerHTML::foldPreprocessor() const
-{
-    return fold_preproc;
-}
-
-
 // Set if preprocessor blocks can be folded.
 void QsciLexerHTML::setFoldPreprocessor(bool fold)
 {
@@ -1158,13 +1178,6 @@ void QsciLexerHTML::setPreprocProp()
 }
 
 
-// Return true if script comments can be folded.
-bool QsciLexerHTML::foldScriptComments() const
-{
-    return fold_script_comments;
-}
-
-
 // Set if script comments can be folded.
 void QsciLexerHTML::setFoldScriptComments(bool fold)
 {
@@ -1181,13 +1194,6 @@ void QsciLexerHTML::setScriptCommentsProp()
 }
 
 
-// Return true if script heredocs can be folded.
-bool QsciLexerHTML::foldScriptHeredocs() const
-{
-    return fold_script_heredocs;
-}
-
-
 // Set if script heredocs can be folded.
 void QsciLexerHTML::setFoldScriptHeredocs(bool fold)
 {
@@ -1202,3 +1208,35 @@ void QsciLexerHTML::setScriptHeredocsProp()
 {
     emit propertyChanged("fold.hypertext.heredoc",(fold_script_heredocs ? "1" : "0"));
 }
+
+
+// Set if Django templates are supported.
+void QsciLexerHTML::setDjangoTemplates(bool enable)
+{
+    django_templates = enable;
+
+    setDjangoProp();
+}
+
+
+// Set the "lexer.html.django" property.
+void QsciLexerHTML::setDjangoProp()
+{
+    emit propertyChanged("lexer.html.django", (django_templates ? "1" : "0"));
+}
+
+
+// Set if Mako templates are supported.
+void QsciLexerHTML::setMakoTemplates(bool enable)
+{
+    mako_templates = enable;
+
+    setMakoProp();
+}
+
+
+// Set the "lexer.html.mako" property.
+void QsciLexerHTML::setMakoProp()
+{
+    emit propertyChanged("lexer.html.mako", (mako_templates ? "1" : "0"));
+}
