@@ -1,6 +1,6 @@
 // This module implements the QsciLexerLua class.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -119,6 +114,7 @@ QColor QsciLexerLua::defaultColor(int style) const
         return QColor(0x7f,0x00,0x7f);
 
     case Preprocessor:
+    case Label:
         return QColor(0x7f,0x7f,0x00);
 
     case Operator:
@@ -152,6 +148,8 @@ QFont QsciLexerLua::defaultFont(int style) const
     case LiteralString:
 #if defined(Q_OS_WIN)
         f = QFont("Comic Sans MS",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Comic Sans MS", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
@@ -275,6 +273,21 @@ QString QsciLexerLua::description(int style) const
 
     case CoroutinesIOSystemFacilities:
         return tr("Coroutines, i/o and system facilities");
+
+    case KeywordSet5:
+        return tr("User defined 1");
+
+    case KeywordSet6:
+        return tr("User defined 2");
+
+    case KeywordSet7:
+        return tr("User defined 3");
+
+    case KeywordSet8:
+        return tr("User defined 4");
+
+    case Label:
+        return tr("Label");
     }
 
     return QString();
