@@ -1,6 +1,6 @@
 // This module implements the "official" low-level API.
 //
-// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -101,9 +101,8 @@ QsciScintillaBase::QsciScintillaBase(QWidget *parent, const char *name,
 
     setFocusPolicy(WheelFocus);
 
-// FIXME: QMacPasteboardMime isn't in Qt v5 yet.
-#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC) && QT_VERSION < 0x050000
-    RectangularPasteboardMime::initialise();
+#if (QT_VERSION >= 0x040200 && QT_VERSION < 0x050000 && defined(Q_OS_MAC)) || (QT_VERSION >= 0x050200 && defined(Q_OS_OSX))
+    initialiseRectangularPasteboardMime();
 #endif
 
     sci = new QsciScintillaQt(this);
@@ -402,16 +401,35 @@ void QsciScintillaBase::contextMenuEvent(QContextMenuEvent *e)
 
 
 // Re-implemented to tell the widget it has the focus.
-void QsciScintillaBase::focusInEvent(QFocusEvent *)
+void QsciScintillaBase::focusInEvent(QFocusEvent *e)
 {
     sci->SetFocusState(true);
+
+#if QT_VERSION >= 0x050000
+    clickCausedFocus = (e->reason() == Qt::MouseFocusReason);
+#endif
+
+    QWidget::focusInEvent(e);
 }
 
 
 // Re-implemented to tell the widget it has lost the focus.
-void QsciScintillaBase::focusOutEvent(QFocusEvent *)
+void QsciScintillaBase::focusOutEvent(QFocusEvent *e)
 {
-    sci->SetFocusState(false);
+    // Only tell Scintilla we have lost focus if the new active window isn't
+    // our auto-completion list.  This is probably only an issue on Linux and
+    // there are still problems because subsequent focus out events don't
+    // always seem to get generated (at least with Qt5).
+
+    if (e->reason() == Qt::ActiveWindowFocusReason)
+    {
+        QWidget *aw = QApplication::activeWindow();
+
+        if (!aw || aw->parent() != this || !aw->inherits("QsciSciListBox"))
+            sci->SetFocusState(false);
+    }
+
+    QWidget::focusOutEvent(e);
 }
 
 
@@ -436,7 +454,7 @@ void QsciScintillaBase::handleSelection()
 // Handle key presses.
 void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
 {
-    unsigned key, modifiers = 0;
+    int modifiers = 0;
 
     if (e->state() & Qt::ShiftButton)
         modifiers |= SCMOD_SHIFT;
@@ -450,7 +468,42 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
     if (e->state() & Qt::MetaButton)
         modifiers |= SCMOD_META;
 
-    switch (e->key())
+    int key = commandKey(e->key(), modifiers);
+
+    if (key)
+    {
+        bool consumed = false;
+
+        sci->KeyDownWithModifiers(key, modifiers, &consumed);
+
+        if (consumed)
+        {
+            e->accept();
+            return;
+        }
+    }
+
+    QString text = e->text();
+
+    if (!text.isEmpty() && text[0].isPrint())
+    {
+        ScintillaBytes bytes = textAsBytes(text);
+        sci->AddCharUTF(bytes.data(), bytes.length());
+        e->accept();
+    }
+    else
+    {
+        QWidget::keyPressEvent(e);
+    }
+}
+
+
+// Map a Qt key to a valid Scintilla command key, or 0 if none.
+int QsciScintillaBase::commandKey(int qt_key, int &modifiers)
+{
+    int key;
+
+    switch (qt_key)
     {
     case Qt::Key_Down:
         key = SCK_DOWN;
@@ -528,52 +581,32 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         break;
 
     default:
-        key = e->key();
+        if ((key = qt_key) > 0x7f)
+            key = 0;
     }
 
-    if (key)
-    {
-        bool consumed = false;
-
-        sci->KeyDownWithModifiers(key, modifiers, &consumed);
-
-        if (consumed)
-        {
-            e->accept();
-            return;
-        }
-    }
-
-    QString text = e->text();
-
-    if (!text.isEmpty() && text[0].isPrint())
-    {
-        QCString enc_text;
-
-        if (sci->IsUnicodeMode())
-        {
-            enc_text = text.utf8();
-        }
-        else
-        {
-            static QTextCodec *codec = 0;
-
-            if (!codec)
-                codec = QTextCodec::codecForName("ISO 8859-1");
-
-            enc_text = codec->fromUnicode(text);
-        }
-
-        sci->AddCharUTF(enc_text.data(), enc_text.length());
-        e->accept();
-    }
-    else
-    {
-        QWidget::keyPressEvent(e);
-    }
+    return key;
 }
 
 
+// Encode a QString as bytes.
+QsciScintillaBase::ScintillaBytes QsciScintillaBase::textAsBytes(const QString &text) const
+{
+    if (sci->IsUnicodeMode())
+        return text.utf8();
+
+    return QCString(text.latin1());
+}
+
+
+// Decode bytes as a QString.
+QString QsciScintillaBase::bytesAsText(const char *bytes) const
+{
+    if (sci->IsUnicodeMode())
+        return QString::fromUtf8(bytes);
+
+    return QString::fromLatin1(bytes);
+}
 
 
 // Handle a mouse button double click.
@@ -667,12 +700,29 @@ void QsciScintillaBase::mousePressEvent(QMouseEvent *e)
 // Handle a mouse button releases.
 void QsciScintillaBase::mouseReleaseEvent(QMouseEvent *e)
 {
-    if (sci->HaveMouseCapture() && e->button() == Qt::LeftButton)
+    if (e->button() != Qt::LeftButton)
+        return;
+
+    QSCI_SCI_NAMESPACE(Point) pt(e->x(), e->y());
+
+    if (sci->HaveMouseCapture())
     {
         bool ctrl = e->state() & Qt::ControlButton;
 
-        sci->ButtonUp(QSCI_SCI_NAMESPACE(Point)(e->x(), e->y()), 0, ctrl);
+        sci->ButtonUp(pt, 0, ctrl);
     }
+
+#if QT_VERSION >= 0x050000
+    if (!sci->pdoc->IsReadOnly() && !sci->PointInSelMargin(pt) && qApp->autoSipEnabled())
+    {
+        QStyle::RequestSoftwareInputPanel rsip = QStyle::RequestSoftwareInputPanel(style()->styleHint(QStyle::SH_RequestSoftwareInputPanel));
+
+        if (!clickCausedFocus || rsip == QStyle::RSIP_OnMouseClick)
+            qApp->inputMethod()->show();
+    }
+
+    clickCausedFocus = false;
+#endif
 }
 
 
@@ -773,12 +823,11 @@ void QsciScintillaBase::dropEvent(QDropEvent *e)
 
     rectangular = false;
 
-    s = QSCI_SCI_NAMESPACE(Document)::TransformLineEnds(&len, s, len,
+    std::string dest = QSCI_SCI_NAMESPACE(Document)::TransformLineEnds(s, len,
                 sci->pdoc->eolMode);
 
-    sci->DropAt(sci->posDrop, s, moving, rectangular);
-
-    delete[] s;
+    sci->DropAt(sci->posDrop, dest.c_str(), dest.length(), moving,
+            rectangular);
 
     sci->Redraw();
 }
