@@ -1,6 +1,6 @@
 // This class defines the "official" low-level API.
 //
-// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -33,6 +33,7 @@ extern "C++" {
 #include <qglobal.h>
 
 #include <QAbstractScrollArea>
+#include <QByteArray>
 
 #include <qpoint.h>
 #include <qtimer.h>
@@ -922,6 +923,15 @@ public:
         //!
         SCI_GETALLLINESVISIBLE = 2236,
 
+        //!
+        SCI_FOLDLINE = 2237,
+
+        //!
+        SCI_FOLDCHILDREN = 2238,
+
+        //!
+        SCI_EXPANDCHILDREN = 2239,
+
         //!
         SCI_SETTABINDENTS = 2260,
 
@@ -1762,6 +1772,12 @@ public:
         //!
         SCI_ANNOTATIONGETSTYLEOFFSET = 2551,
 
+        //!
+        SCI_RELEASEALLEXTENDEDSTYLES = 2552,
+
+        //!
+        SCI_ALLOCATEEXTENDEDSTYLES = 2553,
+
         //!
         SCI_SETEMPTYSELECTION = 2556,
 
@@ -1801,6 +1817,9 @@ public:
         //!
         SCI_GETADDITIONALCARETSBLINK = 2568,
 
+        //!
+        SCI_SCROLLRANGE = 2569,
+
         //!
         SCI_SETADDITIONALCARETSVISIBLE = 2608,
 
@@ -2059,9 +2078,48 @@ public:
         //!
         SCI_VCHOMEDISPLAY = 2652,
 
+        //!
+        SCI_GETCARETLINEVISIBLEALWAYS = 2654,
+
+        //!
+        SCI_SETCARETLINEVISIBLEALWAYS = 2655,
+
         //!
         SCI_VCHOMEDISPLAYEXTEND = 2653,
 
+        //!
+        SCI_AUTOCSETORDER = 2660,
+
+        //!
+        SCI_AUTOCGETORDER = 2661,
+
+        //!
+        SCI_FOLDALL = 2662,
+
+        //!
+        SCI_SETAUTOMATICFOLD = 2663,
+
+        //!
+        SCI_GETAUTOMATICFOLD = 2664,
+
+        //!
+        SCI_SETREPRESENTATION = 2665,
+
+        //!
+        SCI_GETREPRESENTATION = 2666,
+
+        //!
+        SCI_CLEARREPRESENTATION = 2667,
+
+        //!
+        SCI_SETMOUSESELECTIONRECTANGULARSWITCH = 2668,
+
+        //!
+        SCI_GETMOUSESELECTIONRECTANGULARSWITCH = 2669,
+
+        //!
+        SCI_POSITIONRELATIVE = 2670,
+
         //!
         SCI_STARTRECORD = 3001,
 
@@ -2123,7 +2181,7 @@ public:
         SCI_DESCRIBEPROPERTY = 4016,
 
         //!
-        SCI_DESCRIBEKEYWORDSETS = 4017
+        SCI_DESCRIBEKEYWORDSETS = 4017,
     };
 
     enum
@@ -2462,6 +2520,7 @@ public:
         INDIC_SQUIGGLELOW = 11,
         INDIC_DOTBOX = 12,
         INDIC_SQUIGGLEPIXMAP = 13,
+        INDIC_COMPOSITIONTHICK = 14,
 
         INDIC_CONTAINER = 8,
         INDIC_MAX = 31,
@@ -2981,6 +3040,18 @@ public:
 
         //! Select the Visual Prolog lexer.
         SCLEX_VISUALPROLOG = 107,
+
+        //! Select the Literal Haskell lexer.
+        SCLEX_LITERATEHASKELL = 108,
+
+        //! Select the Structured Text lexer.
+        SCLEX_STTXT = 109,
+
+        //! Select the KVIrc lexer.
+        SCLEX_KVIRC = 110,
+
+        //! Select the Rust lexer.
+        SCLEX_RUST = 111,
     };
 
     enum
@@ -3007,6 +3078,27 @@ public:
         SC_FONT_SIZE_MULTIPLIER = 100,
     };
 
+    enum
+    {
+        SC_FOLDACTION_CONTRACT = 0,
+        SC_FOLDACTION_EXPAND = 1,
+        SC_FOLDACTION_TOGGLE = 2,
+    };
+
+    enum
+    {
+        SC_AUTOMATICFOLD_SHOW = 0x0001,
+        SC_AUTOMATICFOLD_CLICK = 0x0002,
+        SC_AUTOMATICFOLD_CHANGE = 0x0004,
+    };
+
+    enum
+    {
+        SC_ORDER_PRESORTED = 0,
+        SC_ORDER_PERFORMSORT = 1,
+        SC_ORDER_CUSTOM = 2,
+    };
+
     //! Construct an empty QsciScintillaBase with parent \a parent.
     explicit QsciScintillaBase(QWidget *parent = 0);
 
@@ -3071,6 +3163,9 @@ public:
     void *SendScintillaPtrResult(unsigned int msg) const;
 
 
+    //! \internal
+    static int commandKey(int qt_key, int &modifiers);
+
 signals:
     //! This signal is emitted when text is selected or de-selected.
     //! \a yes is true if text has been selected and false if text has been
@@ -3123,6 +3218,12 @@ signals:
     //!
     void SCN_DWELLSTART(int, int, int);
 
+    //! This signal is emitted when focus is received.
+    void SCN_FOCUSIN();
+
+    //! This signal is emitted when focus is lost.
+    void SCN_FOCUSOUT();
+
     //! This signal is emitted when the user clicks on text in a style with the
     //! hotspot attribute set.
     //! \a position is the position in the text where the click occured.
@@ -3276,7 +3377,8 @@ protected:
     virtual void keyPressEvent(QKeyEvent *e);
 
     //! Re-implemented to handle composed characters.
-    virtual void inputMethodEvent(QInputMethodEvent *e);
+    virtual void inputMethodEvent(QInputMethodEvent *event);
+    virtual QVariant inputMethodQuery(Qt::InputMethodQuery query) const;
 
     //! Re-implemented to handle mouse double-clicks.
     virtual void mouseDoubleClickEvent(QMouseEvent *e);
@@ -3302,6 +3404,18 @@ protected:
     //! \internal This helps to work around some Scintilla bugs.
     void setScrollBars();
 
+    //! \internal Qt4, Qt5 portability.
+    typedef QByteArray ScintillaBytes;
+
+#define ScintillaBytesConstData(b)  (b).constData()
+
+
+    //! \internal Convert a QString to encoded bytes.
+    ScintillaBytes textAsBytes(const QString &text) const;
+
+    //! \internal Convert encoded bytes to a QString.
+    QString bytesAsText(const char *bytes) const;
+
 private slots:
     void handleTimer();
     void handleVSb(int value);
@@ -3315,6 +3429,12 @@ private:
     QsciScintillaQt *sci;
     QPoint triple_click_at;
     QTimer triple_click;
+    int preeditPos;
+    int preeditNrBytes;
+    QString preeditString;
+#if QT_VERSION >= 0x050000
+    bool clickCausedFocus;
+#endif
 
     void acceptAction(QDropEvent *e);
 
