@@ -1,6 +1,6 @@
 // This module implements the "official" low-level API.
 //
-// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -42,6 +42,7 @@
 #include <QMimeData>
 #include <QMouseEvent>
 #include <QPaintEvent>
+#include <QStyle>
 
 #include "ScintillaQt.h"
 
@@ -82,87 +83,18 @@ static QList<QsciScintillaBase *> poolList;
 static const QLatin1String mimeTextPlain("text/plain");
 static const QLatin1String mimeRectangularWin("MSDEVColumnSelect");
 static const QLatin1String mimeRectangular("text/x-qscintilla-rectangular");
-static const QLatin1String utiRectangularMac("com.scintilla.utf16-plain-text.rectangular");
 
-// FIXME: QMacPasteboardMime isn't in Qt v5 yet.
-#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC) && QT_VERSION < 0x050000
-#include <QMacPasteboardMime>
-
-class RectangularPasteboardMime : public QMacPasteboardMime
-{
-public:
-    RectangularPasteboardMime() : QMacPasteboardMime(MIME_ALL)
-    {
-    }
-
-    bool canConvert(const QString &mime, QString flav)
-    {
-        return mime == mimeRectangular && flav == utiRectangularMac;
-    }
-
-    QList<QByteArray> convertFromMime(const QString &, QVariant data, QString)
-    {
-        QList<QByteArray> converted;
-
-        converted.append(data.toByteArray());
-
-        return converted;
-    }
-
-    QVariant convertToMime(const QString &, QList<QByteArray> data, QString)
-    {
-        QByteArray converted;
-
-        foreach (QByteArray i, data)
-        {
-            converted += i;
-        }
-
-        return QVariant(converted);
-    }
-
-    QString convertorName()
-    {
-        return QString("QScintillaRectangular");
-    }
-
-    QString flavorFor(const QString &mime)
-    {
-        if (mime == mimeRectangular)
-            return QString(utiRectangularMac);
-
-        return QString();
-    }
-
-    QString mimeFor(QString flav)
-    {
-        if (flav == utiRectangularMac)
-            return QString(mimeRectangular);
-
-        return QString();
-    }
-
-    static void initialise()
-    {
-        if (!instance)
-        {
-            instance = new RectangularPasteboardMime();
-
-            qRegisterDraggedTypes(QStringList(utiRectangularMac));
-        }
-    }
-
-private:
-    static RectangularPasteboardMime *instance;
-};
-
-RectangularPasteboardMime *RectangularPasteboardMime::instance;
+#if (QT_VERSION >= 0x040200 && QT_VERSION < 0x050000 && defined(Q_OS_MAC)) || (QT_VERSION >= 0x050200 && defined(Q_OS_OSX))
+extern void initialiseRectangularPasteboardMime();
 #endif
 
 
 // The ctor.
 QsciScintillaBase::QsciScintillaBase(QWidget *parent)
-    : QAbstractScrollArea(parent)
+    : QAbstractScrollArea(parent), preeditPos(-1), preeditNrBytes(0)
+#if QT_VERSION >= 0x050000
+        , clickCausedFocus(false)
+#endif
 {
     connect(verticalScrollBar(), SIGNAL(valueChanged(int)),
             SLOT(handleVSb(int)));
@@ -173,6 +105,13 @@ QsciScintillaBase::QsciScintillaBase(QWidget *parent)
     setAcceptDrops(true);
     setFocusPolicy(Qt::WheelFocus);
     setAttribute(Qt::WA_KeyCompression);
+    setAttribute(Qt::WA_InputMethodEnabled);
+#if QT_VERSION >= 0x050000
+    setInputMethodHints(
+            Qt::ImhNoAutoUppercase|Qt::ImhNoPredictiveText|Qt::ImhMultiLine);
+#elif QT_VERSION >= 0x040600
+    setInputMethodHints(Qt::ImhNoAutoUppercase|Qt::ImhNoPredictiveText);
+#endif
 
     viewport()->setBackgroundRole(QPalette::Base);
     viewport()->setMouseTracking(true);
@@ -180,9 +119,8 @@ QsciScintillaBase::QsciScintillaBase(QWidget *parent)
 
     triple_click.setSingleShot(true);
 
-// FIXME: QMacPasteboardMime isn't in Qt v5 yet.
-#if QT_VERSION >= 0x040200 && defined(Q_OS_MAC) && QT_VERSION < 0x050000
-    RectangularPasteboardMime::initialise();
+#if (QT_VERSION >= 0x040200 && QT_VERSION < 0x050000 && defined(Q_OS_MAC)) || (QT_VERSION >= 0x050200 && defined(Q_OS_OSX))
+    initialiseRectangularPasteboardMime();
 #endif
 
     sci = new QsciScintillaQt(this);
@@ -384,16 +322,35 @@ void QsciScintillaBase::contextMenuEvent(QContextMenuEvent *e)
 
 
 // Re-implemented to tell the widget it has the focus.
-void QsciScintillaBase::focusInEvent(QFocusEvent *)
+void QsciScintillaBase::focusInEvent(QFocusEvent *e)
 {
     sci->SetFocusState(true);
+
+#if QT_VERSION >= 0x050000
+    clickCausedFocus = (e->reason() == Qt::MouseFocusReason);
+#endif
+
+    QAbstractScrollArea::focusInEvent(e);
 }
 
 
 // Re-implemented to tell the widget it has lost the focus.
-void QsciScintillaBase::focusOutEvent(QFocusEvent *)
+void QsciScintillaBase::focusOutEvent(QFocusEvent *e)
 {
-    sci->SetFocusState(false);
+    // Only tell Scintilla we have lost focus if the new active window isn't
+    // our auto-completion list.  This is probably only an issue on Linux and
+    // there are still problems because subsequent focus out events don't
+    // always seem to get generated (at least with Qt5).
+
+    if (e->reason() == Qt::ActiveWindowFocusReason)
+    {
+        QWidget *aw = QApplication::activeWindow();
+
+        if (!aw || aw->parent() != this || !aw->inherits("QsciSciListBox"))
+            sci->SetFocusState(false);
+    }
+
+    QAbstractScrollArea::focusOutEvent(e);
 }
 
 
@@ -418,7 +375,7 @@ void QsciScintillaBase::handleSelection()
 // Handle key presses.
 void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
 {
-    unsigned key, modifiers = 0;
+    int modifiers = 0;
 
     if (e->modifiers() & Qt::ShiftModifier)
         modifiers |= SCMOD_SHIFT;
@@ -432,7 +389,42 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
     if (e->modifiers() & Qt::MetaModifier)
         modifiers |= SCMOD_META;
 
-    switch (e->key())
+    int key = commandKey(e->key(), modifiers);
+
+    if (key)
+    {
+        bool consumed = false;
+
+        sci->KeyDownWithModifiers(key, modifiers, &consumed);
+
+        if (consumed)
+        {
+            e->accept();
+            return;
+        }
+    }
+
+    QString text = e->text();
+
+    if (!text.isEmpty() && text[0].isPrint())
+    {
+        ScintillaBytes bytes = textAsBytes(text);
+        sci->AddCharUTF(bytes.data(), bytes.length());
+        e->accept();
+    }
+    else
+    {
+        QAbstractScrollArea::keyPressEvent(e);
+    }
+}
+
+
+// Map a Qt key to a valid Scintilla command key, or 0 if none.
+int QsciScintillaBase::commandKey(int qt_key, int &modifiers)
+{
+    int key;
+
+    switch (qt_key)
     {
     case Qt::Key_Down:
         key = SCK_DOWN;
@@ -510,60 +502,31 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         break;
 
     default:
-        key = e->key();
+        if ((key = qt_key) > 0x7f)
+            key = 0;
     }
 
-    if (key)
-    {
-        bool consumed = false;
-
-        sci->KeyDownWithModifiers(key, modifiers, &consumed);
-
-        if (consumed)
-        {
-            e->accept();
-            return;
-        }
-    }
-
-    QString text = e->text();
-
-    if (!text.isEmpty() && text[0].isPrint())
-    {
-        QByteArray enc_text;
-
-        if (sci->IsUnicodeMode())
-        {
-            enc_text = text.toUtf8();
-        }
-        else
-        {
-            static QTextCodec *codec = 0;
-
-            if (!codec)
-                codec = QTextCodec::codecForName("ISO 8859-1");
-
-            enc_text = codec->fromUnicode(text);
-        }
-
-        sci->AddCharUTF(enc_text.data(), enc_text.length());
-        e->accept();
-    }
-    else
-    {
-        QAbstractScrollArea::keyPressEvent(e);
-    }
+    return key;
 }
 
 
-// Handle composed characters.  Note that this is the minumum needed to retain
-// the QScintilla v1 functionality.
-void QsciScintillaBase::inputMethodEvent(QInputMethodEvent *e)
+// Encode a QString as bytes.
+QsciScintillaBase::ScintillaBytes QsciScintillaBase::textAsBytes(const QString &text) const
 {
-    QByteArray utf8 = e->commitString().toUtf8();
+    if (sci->IsUnicodeMode())
+        return text.toUtf8();
 
-    sci->AddCharUTF(utf8.data(), utf8.length());
-    e->accept();
+    return text.toLatin1();
+}
+
+
+// Decode bytes as a QString.
+QString QsciScintillaBase::bytesAsText(const char *bytes) const
+{
+    if (sci->IsUnicodeMode())
+        return QString::fromUtf8(bytes);
+
+    return QString::fromLatin1(bytes);
 }
 
 
@@ -630,10 +593,10 @@ void QsciScintillaBase::mousePressEvent(QMouseEvent *e)
         // the modifier to be configured.
         bool shift = e->modifiers() & Qt::ShiftModifier;
         bool ctrl = e->modifiers() & Qt::ControlModifier;
-#if defined(Q_WS_X11)
-        bool alt = ctrl;
-#else
+#if defined(Q_OS_MAC) || defined(Q_OS_WIN)
         bool alt = e->modifiers() & Qt::AltModifier;
+#else
+        bool alt = ctrl;
 #endif
 
         sci->ButtonDown(pt, clickTime, shift, ctrl, alt);
@@ -658,12 +621,29 @@ void QsciScintillaBase::mousePressEvent(QMouseEvent *e)
 // Handle a mouse button releases.
 void QsciScintillaBase::mouseReleaseEvent(QMouseEvent *e)
 {
-    if (sci->HaveMouseCapture() && e->button() == Qt::LeftButton)
+    if (e->button() != Qt::LeftButton)
+        return;
+
+    QSCI_SCI_NAMESPACE(Point) pt(e->x(), e->y());
+
+    if (sci->HaveMouseCapture())
     {
         bool ctrl = e->modifiers() & Qt::ControlModifier;
 
-        sci->ButtonUp(QSCI_SCI_NAMESPACE(Point)(e->x(), e->y()), 0, ctrl);
+        sci->ButtonUp(pt, 0, ctrl);
     }
+
+#if QT_VERSION >= 0x050000
+    if (!sci->pdoc->IsReadOnly() && !sci->PointInSelMargin(pt) && qApp->autoSipEnabled())
+    {
+        QStyle::RequestSoftwareInputPanel rsip = QStyle::RequestSoftwareInputPanel(style()->styleHint(QStyle::SH_RequestSoftwareInputPanel));
+
+        if (!clickCausedFocus || rsip == QStyle::RSIP_OnMouseClick)
+            qApp->inputMethod()->show();
+    }
+
+    clickCausedFocus = false;
+#endif
 }
 
 
@@ -747,12 +727,11 @@ void QsciScintillaBase::dropEvent(QDropEvent *e)
     len = text.length();
     s = text.data();
 
-    s = QSCI_SCI_NAMESPACE(Document)::TransformLineEnds(&len, s, len,
+    std::string dest = QSCI_SCI_NAMESPACE(Document)::TransformLineEnds(s, len,
                 sci->pdoc->eolMode);
 
-    sci->DropAt(sci->posDrop, s, moving, rectangular);
-
-    delete[] s;
+    sci->DropAt(sci->posDrop, dest.c_str(), dest.length(), moving,
+            rectangular);
 
     sci->Redraw();
 }
