@@ -13,6 +13,7 @@
 // PLAT_GTK_WIN32 is defined additionally when running PLAT_GTK under Win32
 // PLAT_WIN = Win32 API on Win32 OS
 // PLAT_WX is wxWindows on any supported platform
+// PLAT_TK = Tcl/TK on Linux or Win32
 
 #define PLAT_GTK 0
 #define PLAT_GTK_WIN32 0
@@ -22,7 +23,8 @@
 #define PLAT_WX  0
 #define PLAT_QT 0
 #define PLAT_FOX 0
-#define PLAT_NCURSES 0
+#define PLAT_CURSES 0
+#define PLAT_TK 0
 
 #if defined(FOX)
 #undef PLAT_FOX
@@ -32,11 +34,11 @@
 #undef PLAT_WX
 #define PLAT_WX  1
 
-#elif defined(NCURSES)
-#undef PLAT_NCURSES
-#define PLAT_NCURSES 1
+#elif defined(CURSES)
+#undef PLAT_CURSES
+#define PLAT_CURSES 1
 
-#elif defined(QT)
+#elif defined(SCINTILLA_QT)
 #undef PLAT_QT
 #define PLAT_QT 1
 
@@ -48,6 +50,10 @@ QT_END_NAMESPACE
 // This is needed to work around an HP-UX bug with Qt4.
 #include <qnamespace.h>
 
+#elif defined(TK)
+#undef PLAT_TK
+#define PLAT_TK 1
+
 #elif defined(GTK)
 #undef PLAT_GTK
 #define PLAT_GTK 1
@@ -127,19 +133,19 @@ public:
 
 	// Other automatically defined methods (assignment, copy constructor, destructor) are fine
 
-	bool operator==(PRectangle &rc) {
+	bool operator==(PRectangle &rc) const {
 		return (rc.left == left) && (rc.right == right) &&
 			(rc.top == top) && (rc.bottom == bottom);
 	}
-	bool Contains(Point pt) {
+	bool Contains(Point pt) const {
 		return (pt.x >= left) && (pt.x <= right) &&
 			(pt.y >= top) && (pt.y <= bottom);
 	}
-	bool Contains(PRectangle rc) {
+	bool Contains(PRectangle rc) const {
 		return (rc.left >= left) && (rc.right <= right) &&
 			(rc.top >= top) && (rc.bottom <= bottom);
 	}
-	bool Intersects(PRectangle other) {
+	bool Intersects(PRectangle other) const {
 		return (right > other.left) && (left < other.right) &&
 			(bottom > other.top) && (top < other.bottom);
 	}
@@ -149,9 +155,9 @@ public:
 		right += xDelta;
 		bottom += yDelta;
 	}
-	XYPOSITION Width() { return right - left; }
-	XYPOSITION Height() { return bottom - top; }
-	bool Empty() {
+	XYPOSITION Width() const { return right - left; }
+	XYPOSITION Height() const { return bottom - top; }
+	bool Empty() const {
 		return (Height() <= 0) || (Width() <= 0);
 	}
 };
@@ -207,15 +213,15 @@ public:
 		return co;
 	}
 
-	unsigned int GetRed() {
+	unsigned int GetRed() const {
 		return co & 0xff;
 	}
 
-	unsigned int GetGreen() {
+	unsigned int GetGreen() const {
 		return (co >> 8) & 0xff;
 	}
 
-	unsigned int GetBlue() {
+	unsigned int GetBlue() const {
 		return (co >> 16) & 0xff;
 	}
 };
@@ -355,22 +361,10 @@ typedef void (*CallBackAction)(void*);
 class Window {
 protected:
 	WindowID wid;
-#if PLAT_MACOSX
-	void *windowRef;
-	void *control;
-#endif
 public:
 	Window() : wid(0), cursorLast(cursorInvalid) {
-#if PLAT_MACOSX
-	  windowRef = 0;
-	  control = 0;
-#endif
 	}
 	Window(const Window &source) : wid(source.wid), cursorLast(cursorInvalid) {
-#if PLAT_MACOSX
-	  windowRef = 0;
-	  control = 0;
-#endif
 	}
 	virtual ~Window();
 	Window &operator=(WindowID wid_) {
@@ -393,10 +387,6 @@ public:
 	void SetCursor(Cursor curs);
 	void SetTitle(const char *s);
 	PRectangle GetMonitorRect(Point pt);
-#if PLAT_MACOSX
-	void SetWindow(void *ref) { windowRef = ref; }
-	void SetControl(void *_control) { control = _control; }
-#endif
 private:
 	Cursor cursorLast;
 };
@@ -538,9 +528,7 @@ public:
 #endif
 
 #if defined(__GNUC__) && defined(SCINTILLA_QT)
-#pragma GCC diagnostic ignored "-Wmissing-braces"
 #pragma GCC diagnostic ignored "-Wmissing-field-initializers"
-#pragma GCC diagnostic ignored "-Wchar-subscripts"
 #endif
 
 #endif
